C                                                            <HTML><PRE>
C Program GRDMIGR to perform common-source prestack Kirchhoff migration
C of 1-component or 3-component elastic seismograms using the scalar
C imaging functional
C
C Version: 8.10
C Date: 2022, November 28
C
C Coded by: Ludek Klimes
C     Department of Geophysics, Charles University,
C     Ke Karlovu 3, 121 16 Praha 2, Czech Republic,
C     http://sw3d.cz/staff/klimes.htm
C
C.......................................................................
C                                                    <A NAME="data"></A>
C Description of data files:
C
C Input data read from the standard input device (*):
C     The data are read by the list directed input (free format) and
C     consist of a single string 'SEP':
C     'SEP'...String in apostrophes containing the name of the input
C             SEP parameter or history file with the input data.
C     No default, 'SEP' must be specified and cannot be blank.
C
C                                                     <A NAME="SEP"></A>
C Input data file 'SEP':
C     File 'SEP' has the form of the <A HREF="../forms/sep.htm">SEP</A>
C     parameter file.  The parameters, which do not differ from their
C     defaults, need not be specified in file 'SEP'.
C Names of the input and output files:
C     MIGR='string'... Name of the input ASCII file containing the
C             names of files with grid values of travel times and
C             amplitudes.
C             Default: MIGR='migr.dat'
C     GRDSEIS='string'... Name of the input file with 1 component of
C             seismograms corresponding to source ISRC.
C             File contains NSEIS*NREC real values.
C             The first NSEIS values correspond to receiver IREC,
C             the last NSEIS values correspond to receiver IREC+NREC-1.
C             If GRDSEIS=' ', 3-component seismograms are migrated
C             instead.
C             Default: GRDSEIS='grdseis.out'
C     GRDSEIS1='string', GRDSEIS2='string', GRDSEIS3='string'... Names
C             of the input files with 3 components of seismograms
C             corresponding to source ISRC.
C             Each file contains NSEIS*NREC real values.
C             The first NSEIS values correspond to receiver IREC,
C             the last NSEIS values correspond to receiver IREC+NREC-1.
C             The parameters are ignored if GRDSEIS.NE.' '.
C             Default: GRDSEIS1='grdseis1.out', GRDSEIS2='grdseis2.out',
C                      GRDSEIS3='grdseis3.out'
C     GRDMIGR='string'... Name of the output file containing the
C             grid values of the migrated image.
C             Default: GRDMIGR='grdmigr.out'
C     For general description of the files with gridded data refer
C     to file <A HREF="../forms/forms.htm#GRD">forms.htm</A>.
C Data specifying dimensions of the input and output grids:
C     N1=positive integer... Number of gridpoints along the X1 axis
C             (inner loop).
C             Default: N1=1
C     N2=positive integer... Number of gridpoints along the X2 axis
C             (intermediate loop).
C             Default: N2=1
C     N3=positive integer... Number of gridpoints along the X3 axis
C             (outer loop).
C             Default: N3=1
C     NSEIS=positive integer... Number of points of a seismogram.
C             Default: NSEIS=1
C     OSEIS=real... Time of the first sample of a seismogram.
C             Default: OSEIS=0.
C     DSEIS=real... Sampling interval of a seismogram.
C             Default: DSEIS=1.
C     ISRC=positive integer... Index of the line in file MIGR
C             corresponding to the source.
C             Default: ISRC=1
C     IREC=positive integer... Index of the line in file MIGR
C             corresponding to the first receiver.
C             Default: IREC=1
C     NREC=positive integer... Number of receivers.
C             Receivers IREC to IREC+NREC-1 are thus considered.
C             Default: NREC=1
C     TAPER=real... Cosine windowing of seismograms.  The width of
C             cosine windows is TAPER intervals between receivers.
C             The first cosine window starts at receiver IREC-1, the
C             second cosine window ends at receiver IREC+NREC.
C             Default: TAPER=0.
C Optional parameters specifying the form of the quantities
C written in the output formatted files:
C     MINDIG,MAXDIG=positive integers ... See the description in file
C             <A HREF="forms.for#MAXDIG" TYPE="text/html">forms.for</A>.
C     NUMLIN=positive integer ... Number of reals or integers in one
C             line of the output file. See the description in file
C             <A HREF="forms.for#NUMLIN" TYPE="text/html">forms.for</A>.
C Form of the input/output files:
C     FORM='string'... Form of the input/output files: either
C             'FORMATTED' or 'UNFORMATTED' (case insensitive).
C             Default: FORM='FORMATTED'
C     FORMW='string'... Form of the output file. If both FORM and FORMW
C             are specified, FORMW is used for output files.
C             Default: FORMW=FORM
C     FORMR='string'... Form of the input file. If both FORM and FORMR
C             are specified, FORMR is used for output files.
C             Default: FORMR=FORM
C Value of undefined quantities:
C     UNDEF=real... The value to be used for undefined real quantities.
C             Default: UNDEF=undefined value used in <A HREF="../forms/forms.for#UARRAY" TYPE="text/html">forms.for</A>
C
C                                                    <A NAME="MIGR"></A>
C Input formatted file MIGR:
C     For each surface point, the following line:
C (1) 'GRDNUM','GRDTT','GRDAMP','GRDAKI',/
C     or
C     'GRDNUM','GRDTT','GRDAMP','GRDAKI','GRDAK1','GRDAK2','GRDAK3',/
C     'GRDNUM'... Any string.  Reserved for future extension.
C     'GRDTT'... Gridded travel times from the surface point.
C     'GRDAMP'... Gridded ray-theory amplitudes from the surface point.
C     'GRDAKI'... Gridded amplitudes, modified for the Kirchhoff
C             integral, corresponding to the surface point.
C             Used only if GRDSEIS.NE.' '.
C     'GRDAK1','GRDAK2','GRDAK3'... Three components of gridded
C             vectorial amplitude corresponding to the three seismogram
C             components at the surface point, modified for the
C             Kirchhoff integral.  Output of program <A HREF="../forms/grdampkm.for" TYPE="text/html">grdampkm.for</A>.
C             Used only if GRDSEIS=' '.
C
C=======================================================================
C
C Common block /RAMC/:
      INCLUDE 'ram.inc'
C     <A HREF="ram.inc" TYPE="text/html">ram.inc</A>
C
C Subroutines and external functions required:
      EXTERNAL ERROR,RSEP1,RSEP3T,RSEP3I,RSEP3R,RARRAY,WARRAY
C     ERROR... File <A HREF="error.for" TYPE="text/html">error.for/A>.
C     RSEP1,RSEP3T,RSEP3I,RSEP3R... File <A HREF="sep.for" TYPE="text/html">sep.for</A>.
C     RARRAY,WARRAY... File <A HREF="forms.for" TYPE="text/html">forms.for</A>.
C
C.......................................................................
C
C     Filenames and parameters:
      CHARACTER*80 FSEP,FNAMES,FSEIS1,FSEIS2,FSEIS3,FMIGR,FILNUM,FILTT
      CHARACTER*80 FILAMP,FILAKI,FILAK1,FILAK2,FILAK3
      INTEGER LU1,LU2
      PARAMETER (LU1=1,LU2=2)
C
C     Input data:
      INTEGER N1,N2,N3,NSEIS,ISRC,IREC,NREC
      REAL OSEIS,DSEIS,TAPER
C
C     Memory allocation:
      INTEGER ISEIS1,ISEIS2,ISEIS3,IMIGR,ITT1,IAMP1
      INTEGER ITT2,IAMP21,IAMP22,IAMP23,NRAM
C
C     Other variables:
      LOGICAL L3
      INTEGER N123,I1,I2,I3,I4,I,IT,JT,JSEIS1,JSEIS2,JSEIS3
      REAL WEIGHT,AMP1,AMP21,AMP22,AMP23,A,TT,DT,DT1
C
C-----------------------------------------------------------------------
C
C     Reading input SEP parameter file:
      WRITE(*,'(A)') '+GRDMIGR: Enter input filename: '
      FSEP=' '
      READ(*,*) FSEP
      IF (FSEP.EQ.' ') THEN
C       <A NAME="error01"></A><FONT COLOR="RED">GRDMIGR-01</FONT>
        CALL ERROR('GRDMIGR-01: SEP file not given')
C       Input file in the form of the SEP (Stanford Exploration Project)
C       parameter or history file must be specified.
C       There is no default filename.
      END IF
      CALL RSEP1(LU1,FSEP)
      WRITE(*,'(A)') '+GRDMIGR: Working ...           '
C
C     Reading input parameters from the SEP file:
      CALL RSEP3T('MIGR',FNAMES,'migr.dat')
      CALL RSEP3T('GRDSEIS',FSEIS1,'grdseis.out')
      IF(FSEIS1.EQ.' ') THEN
C       3-component migration
        L3=.TRUE.
        CALL RSEP3T('GRDSEIS1',FSEIS1,'grdseis1.out')
        CALL RSEP3T('GRDSEIS2',FSEIS2,'grdseis2.out')
        CALL RSEP3T('GRDSEIS3',FSEIS3,'grdseis3.out')
      ELSE
C       1-component migration
        L3=.FALSE.
      END IF
      CALL RSEP3T('GRDMIGR',FMIGR,'grdmigr.out')
      CALL RSEP3I('N1',N1,1)
      CALL RSEP3I('N2',N2,1)
      CALL RSEP3I('N3',N3,1)
      N123=N1*N2*N3
      CALL RSEP3I('NSEIS',NSEIS,1)
      CALL RSEP3R('OSEIS',OSEIS,0.)
      CALL RSEP3R('DSEIS',DSEIS,1.)
      IF (DSEIS.LE.0.) THEN
C       <A NAME="error03"></A><FONT COLOR="RED">GRDMIGR-03</FONT>
        CALL ERROR('GRDMIGR-03: Nonpositive parameter DSEIS')
C       Input SEP parameter DSEIS must be positive.
      END IF
      CALL RSEP3I('ISRC',ISRC,1)
      CALL RSEP3I('IREC',IREC,1)
      CALL RSEP3I('NREC',NREC,1)
      CALL RSEP3R('TAPER',TAPER,0.)
C
C     Memory allocation:
      ISEIS1=0
      IF(L3) THEN
        ISEIS2=ISEIS1+NSEIS*NREC
        ISEIS3=ISEIS2+NSEIS*NREC
        IMIGR =ISEIS3+NSEIS*NREC
      ELSE
        IMIGR =ISEIS1+NSEIS*NREC
      END IF
      ITT1  =IMIGR +N123
      IAMP1 =ITT1  +N123
      ITT2  =IAMP1 +N123
      IAMP21=ITT2  +N123
      IF(L3) THEN
        IAMP22=IAMP21+N123
        IAMP23=IAMP22+N123
        NRAM  =IAMP23+N123
      ELSE
        NRAM  =IAMP21+N123
      END IF
      IF(NRAM.GT.MRAM) THEN
C       <A NAME="error02"></A><FONT COLOR="RED">GRDMIGR-02</FONT>
        CALL ERROR('GRDMIGR-02: Too small array RAM(MRAM)')
C       Array RAM(MRAM) allocated in include file 'ram.inc' is too small
C       to contain both input and output grids.
C       You may wish to increase the dimension MRAM in file
C       <A HREF="ram.inc" TYPE="text/html">ram.inc</A>.
      END IF
C
C     Reading the seismograms:
      CALL RARRAY(LU2,FSEIS1,.TRUE.,0.,NSEIS*NREC,RAM(ISEIS1+1))
      IF(L3) THEN
        CALL RARRAY(LU2,FSEIS2,.TRUE.,0.,NSEIS*NREC,RAM(ISEIS2+1))
        CALL RARRAY(LU2,FSEIS3,.TRUE.,0.,NSEIS*NREC,RAM(ISEIS3+1))
      END IF
C
C     Initializing the image:
      I=0
      DO 13 I3=1,N3
        DO 12 I2=1,N2
          DO 11 I1=1,N1
            I=I+1
            RAM(IMIGR+I)=0.
   11     CONTINUE
   12   CONTINUE
   13 CONTINUE
C
C     Reading the source grid:
      OPEN(LU1,FILE=FNAMES,FORM='formatted',STATUS='old')
      DO 14 I4=1,ISRC
        READ(LU1,*) FILNUM,FILTT,FILAMP,FILAKI,FILAK1,FILAK2,FILAK3
   14 CONTINUE
      CALL RARRAY(LU2,FILTT ,.TRUE.,0.,N123,RAM(ITT1 +1))
      CALL RARRAY(LU2,FILAMP,.TRUE.,0.,N123,RAM(IAMP1+1))
C
C     Finding the first receiver grid:
      REWIND(LU1)
      DO 15 I4=1,IREC-1
        READ(LU1,*) FILNUM,FILTT,FILAMP,FILAKI,FILAK1,FILAK2,FILAK3
   15 CONTINUE
C
C     Loop over the receivers:
      JSEIS1=ISEIS1-NSEIS+1
      IF(L3) THEN
        JSEIS2=ISEIS2-NSEIS+1
        JSEIS3=ISEIS3-NSEIS+1
      END IF
      DO 24 I4=1,NREC
        JSEIS1=JSEIS1+NSEIS
        IF(L3) THEN
          JSEIS2=JSEIS2+NSEIS
          JSEIS3=JSEIS3+NSEIS
        END IF
        READ(LU1,*) FILNUM,FILTT,FILAMP,FILAKI,FILAK1,FILAK2,FILAK3
        CALL RARRAY(LU2,FILTT ,.TRUE.,0.,N123,RAM(ITT2 +1))
        IF(L3) THEN
          CALL RARRAY(LU2,FILAK1,.TRUE.,0.,N123,RAM(IAMP21+1))
          CALL RARRAY(LU2,FILAK2,.TRUE.,0.,N123,RAM(IAMP22+1))
          CALL RARRAY(LU2,FILAK3,.TRUE.,0.,N123,RAM(IAMP23+1))
        ELSE
          CALL RARRAY(LU2,FILAKI,.TRUE.,0.,N123,RAM(IAMP21+1))
        END IF
C
C       Cosine window:
        WEIGHT=1.
        IF(FLOAT(I4).LT.TAPER) THEN
          WEIGHT=0.5-0.5*COS(3.14159*FLOAT(I4)/TAPER)
        END IF
        IF(FLOAT(NREC-I4+1).LT.TAPER) THEN
          WEIGHT=(0.5-0.5*COS(3.14159*FLOAT(NREC-I4+1)/TAPER))*WEIGHT
        END IF
C
C       Loop over the depth points
        I=0
        DO 23 I3=1,N3
          DO 22 I2=1,N2
            DO 21 I1=1,N1
              I=I+1
              A=0.
              AMP1=RAM(IAMP1+I)
              IF(AMP1.GT.0.) THEN
                AMP21=RAM(IAMP21+I)
                IF(L3) THEN
                  AMP22=RAM(IAMP22+I)
                  AMP23=RAM(IAMP23+I)
                END IF
                TT=(RAM(ITT1+I)+RAM(ITT2+I)-OSEIS)/DSEIS
                IT=INT(TT)
                IF(0..LE.TT.AND.IT.LT.NSEIS-1) THEN
                  DT=TT-FLOAT(IT)
                  DT1=1.-DT
                  JT=JSEIS1+IT
                  A=AMP21*(DT1*RAM(JT)+DT*RAM(JT+1))
                  IF(L3) THEN
                    JT=JSEIS2+IT
                    A=AMP22*(DT1*RAM(JT)+DT*RAM(JT+1))+A
                    JT=JSEIS3+IT
                    A=AMP23*(DT1*RAM(JT)+DT*RAM(JT+1))+A
                  END IF
                  A=A/AMP1
                END IF
              END IF
              RAM(IMIGR+I)=RAM(IMIGR+I)+WEIGHT*A
   21       CONTINUE
   22     CONTINUE
   23   CONTINUE
C
   24 CONTINUE
C
C     Writing the migrated image:
      CALL WARRAY(LU2,FMIGR,.FALSE.,0.,.FALSE.,0.,N123,RAM(IMIGR+1))
      CLOSE(LU1)
      WRITE(*,'(A,I8,A)') '+GRDMIGR: Done (source',ISRC,').  '
      STOP
      END
C
C=======================================================================
C
      INCLUDE 'error.for'
C     <A HREF="error.for" TYPE="text/html">error.for</A>
      INCLUDE 'sep.for'
C     <A HREF="sep.for" TYPE="text/html">sep.for</A>
      INCLUDE 'forms.for'
C     <A HREF="forms.for" TYPE="text/html">forms.for</A>
      INCLUDE 'length.for'
C     <A HREF="length.for" TYPE="text/html">length.for</A>
C
C=======================================================================
C                                                                 </PRE>
